//
//  WTCameraControlsViewController.m
//  Native Examples
//
//  Created by Andreas Schacherbauer on 29/07/15.
//  Copyright (c) 2015 Wikitude. All rights reserved.
//

#import "WTCameraControlsViewController.h"

#import <WikitudeNativeSDK/WikitudeNativeSDK.h>

#import "WikitudeLicense.h"
#import "WTCameraControlsModel.h"
#import "StrokedRectangle.h"
#import "ExternalRenderer.h"
#import "WTTargetImageView.h"


@interface WTCameraControlsViewController () <WTWikitudeNativeSDKDelegate, WTImageTrackerDelegate, WTCaptureDeviceManagerDelegate, WTCameraControlsModelDelegate>

@property (nonatomic, strong) WTWikitudeNativeSDK                           *wikitudeSDK;
@property (nonatomic, strong) WTTargetCollectionResource                    *targetCollectionResource;
@property (nonatomic, strong) WTImageTracker                                *imageTracker;

@property (nonatomic, assign) AVCaptureDevicePosition                       currentlyActiveCaptureDevicePosition;

@property (nonatomic, strong) EAGLContext                                   *sharedWikitudeEAGLCameraContext;

@property (nonatomic, copy) WTWikitudeUpdateHandler                         wikitudeUpdateHandler;
@property (nonatomic, copy) WTWikitudeDrawHandler                           wikitudeDrawHandler;

@property (nonatomic, assign) BOOL                                          isTracking;

@property (nonatomic, assign) BOOL                                          isPresentingCameraControls;
@property (nonatomic, strong) WTCameraControlsModel                         *cameraControlsPickerModel;

@property (nonatomic, strong) ExternalRenderer                              *renderer;
@property (nonatomic, strong) StrokedRectangle                              *renderableRectangle;

@property (nonatomic, strong) WTTargetImageView                             *targetImageView;

@end

@implementation WTCameraControlsViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.

    self.cameraControlsPickerModel = [[WTCameraControlsModel alloc] init];
    self.cameraControlsPickerModel.delegate = self;
    self.cameraControlPicker.dataSource = _cameraControlsPickerModel;
    self.cameraControlPicker.delegate = _cameraControlsPickerModel;

    self.renderer = [[ExternalRenderer alloc] init];
    self.renderableRectangle = [[StrokedRectangle alloc] init];    

    self.wikitudeSDK = [[WTWikitudeNativeSDK alloc] initWithRenderingMode:WTRenderingMode_External delegate:self];
    [self.wikitudeSDK setLicenseKey:kWTLicenseKey];

    self.currentlyActiveCaptureDevicePosition = AVCaptureDevicePositionUnspecified;

    self.targetImageView = [[WTTargetImageView alloc] init];
    [self.view addSubview:self.targetImageView];

    NSArray<NSString *> *imageNames = @[@"surfer", @"bike"];
    [self.targetImageView setupViewWithImages:imageNames andTopLayoutGuide:self.topLayoutGuide];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];

    [self.renderer setupRenderingWithLayer:[self.eaglView eaglLayer] OpenGLESRenderingAPIs:[NSOrderedSet orderedSetWithObjects:RenderingAPI_OpenGL_ES_2, nil]];
    [self.renderer startRenderLoopWithRenderBlock:[self renderBlock]];

    [self.wikitudeSDK start:[self startupHandler] completion:^(BOOL isRunning, NSError * __nonnull error) {
        if ( !isRunning ) {
            NSLog(@"Wikitude SDK is not running. Reason: %@", [error localizedDescription]);
        }
        else
        {
            __weak typeof(self) weakSelf = self;
            
            NSURL *imageTrackerResourceURL = [[NSBundle mainBundle] URLForResource:@"magazine" withExtension:@"wtc" subdirectory:@"Assets"];
            self.targetCollectionResource = [self.wikitudeSDK.trackerManager createTargetCollectionResourceFromURL:imageTrackerResourceURL completion:^(BOOL success, NSError * _Nullable error) {
                if ( !success )
                {
                    NSLog(@"Failed to load URL resource. Reason: %@", [error localizedDescription]);
                }
                else
                {
                    weakSelf.wikitudeSDK.captureDeviceManager.delegate = self;
                    weakSelf.imageTracker = [weakSelf.wikitudeSDK.trackerManager createImageTrackerFromTargetCollectionResource:weakSelf.targetCollectionResource delegate:weakSelf configuration:nil];
                }
            }];
        }
    }];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didReceiveApplicationWillResignActiveNotification:) name:UIApplicationWillResignActiveNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didReceiveApplicationDidBecomeActiveNotification:) name:UIApplicationDidBecomeActiveNotification object:nil];

    [self.targetImageView show:YES];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];

    [self.wikitudeSDK stop];
    [self.renderableRectangle releaseProgram];
    [self.renderer stopRenderLoop];
    [self.renderer teardownRendering];
    
    _sharedWikitudeEAGLCameraContext = nil;
    
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)viewWillTransitionToSize:(CGSize)size withTransitionCoordinator:(id<UIViewControllerTransitionCoordinator>)coordinator
{
    [super viewWillTransitionToSize:size withTransitionCoordinator:coordinator];

    [self.wikitudeSDK shouldTransitionToSize:size withTransitionCoordinator:coordinator];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Actions
- (IBAction)handleCameraControlsPresentation:(id)sender {

    [self.cameraControlPicker reloadAllComponents];
    [self.cameraControlsPickerModel updatePickerView:self.cameraControlPicker];

    [self.view layoutIfNeeded];

    self.isPresentingCameraControls = !self.isPresentingCameraControls;
    self.cameraControlHeightConstraint.constant = self.isPresentingCameraControls ? 216 : 0;
    [UIView animateWithDuration:0.5 delay:0.0 usingSpringWithDamping:0.5 initialSpringVelocity:0.7 options:0 animations:^{
        [self.view layoutIfNeeded];
    } completion:nil];
}

#pragma mark - ExternalRenderer render loop
- (ExternalRenderBlock)renderBlock
{
    return ^ (CADisplayLink *displayLink) {
        if ( self.wikitudeUpdateHandler
            &&
            self.wikitudeDrawHandler )
        {
            self.wikitudeUpdateHandler();
            self.wikitudeDrawHandler();
        }

        [self.renderer bindBuffer];

        if ( _isTracking )
        {
            [self.renderableRectangle drawInContext:[self.renderer internalContext]];
        }
    };
}

#pragma mark - Private Methods
- (void (^)(WTNativeSDKStartupConfiguration *startupConfiguration))startupHandler
{
    return ^(WTNativeSDKStartupConfiguration *startupConfiguration) {
        if ( self.currentlyActiveCaptureDevicePosition != AVCaptureDevicePositionUnspecified ) {
            startupConfiguration.captureDevicePosition = self.currentlyActiveCaptureDevicePosition;
        }
        startupConfiguration.captureDeviceResolution = WTCaptureDeviceResolution_AUTO;
        startupConfiguration.targetFrameRate = WTMakeTargetFrameRateAuto();
    };
}

#pragma mark - Notifications
- (void)didReceiveApplicationWillResignActiveNotification:(NSNotification *)notification
{
    [self.wikitudeSDK stop];

    [self.renderer stopRenderLoop];
}

- (void)didReceiveApplicationDidBecomeActiveNotification:(NSNotification *)notification
{
    [self.renderer startRenderLoopWithRenderBlock:[self renderBlock]];

    [self.wikitudeSDK start:[self startupHandler] completion:^(BOOL isRunning, NSError * __nonnull error) {
        if ( !isRunning ) {
            NSLog(@"Wikitude SDK is not running. Reason: %@", [error localizedDescription]);
        }
        else
        {
            self.wikitudeSDK.captureDeviceManager.delegate = self;
        }
    }];
}

#pragma mark - Delegation
#pragma mark WTWikitudeNativeSDKDelegte
- (void)wikitudeNativeSDK:(WTWikitudeNativeSDK * __nonnull)wikitudeNativeSDK didCreatedExternalUpdateHandler:(WTWikitudeUpdateHandler __nonnull)updateHandler
{
    self.wikitudeUpdateHandler = updateHandler;
}

- (void)wikitudeNativeSDK:(WTWikitudeNativeSDK * __nonnull)wikitudeNativeSDK didCreatedExternalDrawHandler:(WTWikitudeDrawHandler __nonnull)drawHandler
{
    self.wikitudeDrawHandler = drawHandler;
}

- (EAGLContext *)eaglContextForVideoCameraInWikitudeNativeSDK:(WTWikitudeNativeSDK * __nonnull)wikitudeNativeSDK
{
    if (!_sharedWikitudeEAGLCameraContext )
    {
        EAGLContext *rendererContext = [self.renderer internalContext];
        self.sharedWikitudeEAGLCameraContext = [[EAGLContext alloc] initWithAPI:[rendererContext API] sharegroup:[rendererContext sharegroup]];
    }
    return self.sharedWikitudeEAGLCameraContext;
}

- (CGRect)eaglViewSizeForExternalRenderingInWikitudeNativeSDK:(WTWikitudeNativeSDK * __nonnull)wikitudeNativeSDK
{
    return self.eaglView.bounds;
}

- (void)wikitudeNativeSDK:(WTWikitudeNativeSDK * __nonnull)wikitudeNativeSDK didEncounterInternalError:(NSError * __nonnull)error
{
    NSLog(@"Internal Wikitude SDK error encounterd. %@", [error localizedDescription]);
}

#pragma mark WTImageTrackerDelegate

- (void)imageTracker:(nonnull WTImageTracker *)imageTracker didRecognizeImage:(nonnull WTImageTarget *)recognizedTarget
{
    NSLog(@"recognized target '%@'", [recognizedTarget name]);
    _isTracking = YES;
    [self.targetImageView hide:YES];
}

- (void)imageTracker:(nonnull WTImageTracker *)imageTracker didTrackImage:(nonnull WTImageTarget *)trackedTarget
{
    [self.renderableRectangle setProjectionMatrix:trackedTarget.projection];
    [self.renderableRectangle setModelViewMatrix:trackedTarget.modelView];
}

- (void)imageTracker:(nonnull WTImageTracker *)imageTracker didLoseImage:(nonnull WTImageTarget *)lostTarget
{
    NSLog(@"lost target '%@'", [lostTarget name]);
    _isTracking = NO;
}

- (void)imageTrackerDidLoadTargets:(nonnull WTImageTracker *)imageTracker
{
    NSLog(@"Image tracker loaded");
}

- (void)imageTracker:(nonnull WTImageTracker *)imageTracker didFailToLoadTargets:(nonnull NSError *)error
{
    NSLog(@"Unable to load image tracker. Reason: %@", [error localizedDescription]);
}

#pragma mark WTCaptureDeviceManager
- (void)captureDeviceManager:(WTCaptureDeviceManager *)captureDeviceManager didSwitchToActiveCaptureDevicePosition:(AVCaptureDevicePosition)activeCaptureDevicePosition
{    
    self.currentlyActiveCaptureDevicePosition = activeCaptureDevicePosition;
}

#pragma mark WTCameraControlsModelDelegate
- (NSMutableArray *)validateAvailableCameraOptions:(NSMutableArray *)availableOptions
{
    if ( ![[self.wikitudeSDK captureDeviceManager] hasTorch] )
    {
        NSString *torchModeOption = @"Torch Mode";
        if ( [availableOptions containsObject:torchModeOption] )
        {
            [availableOptions removeObject:torchModeOption];
        }
    }

    return availableOptions;
}

- (void)cameraControlsModel:(WTCameraControlsModel *)cameraControlsModel didSelectCameraPosition:(AVCaptureDevicePosition)activeCameraPosition
{
    if ( [self.wikitudeSDK captureDeviceManager] ) {
        [[self.wikitudeSDK captureDeviceManager] setActiveCaptureDevicePosition:activeCameraPosition];
    }
}

- (void)cameraControlsModel:(WTCameraControlsModel *)cameraControlsModel didSelectCameraFocusMode:(AVCaptureFocusMode)focusMode
{
    if ( [self.wikitudeSDK captureDeviceManager] ) {
        [[self.wikitudeSDK captureDeviceManager] setFocusMode:focusMode];
    }
}

- (void)cameraControlsModel:(WTCameraControlsModel *)cameraControlsModel didSelectContinuousAutoFocusMode:(AVCaptureAutoFocusRangeRestriction)rangeRestriction
{
    if ( [self.wikitudeSDK captureDeviceManager] ) {
        [[self.wikitudeSDK captureDeviceManager] setAutoFocusRangeRestriction:rangeRestriction];
    }
}

- (void)cameraControlsModel:(WTCameraControlsModel *)cameraControlsModel didSelectCameraFocusDistance:(float)focusDistance
{
    if ( [self.wikitudeSDK captureDeviceManager] ) {
        [[self.wikitudeSDK captureDeviceManager] setFocusDistance:focusDistance];
    }
}

- (void)cameraControlsModel:(WTCameraControlsModel *)cameraControlsModel didSelectTorchMode:(AVCaptureTorchMode)torchMode
{
    if ( [self.wikitudeSDK captureDeviceManager] )
    {
        [[self.wikitudeSDK captureDeviceManager] setTorchMode:torchMode];
    }
}

- (AVCaptureDevicePosition)currentlyActiveCameraPositionForCameraControlsModel:(WTCameraControlsModel *)cameraControlsModel
{
    return [[self.wikitudeSDK captureDeviceManager] activeCaptureDevicePosition];
}

- (AVCaptureFocusMode)currentlyActiveFocusModeForCameraControlsModel:(WTCameraControlsModel *)cameraControlsModel
{
    return [[self.wikitudeSDK captureDeviceManager] focusMode];
}

- (AVCaptureAutoFocusRangeRestriction)currentlyActiveAutoFocusRangeRestrictionForCameraControlsModel:(WTCameraControlsModel *)cameraControlsModel
{
    return [[self.wikitudeSDK captureDeviceManager] autoFocusRangeRestriction];
}

- (float)currentlyActiveCameraFocusDistanceForCameraControlsModel:(WTCameraControlsModel *)cameraControlsModel
{
    return [[self.wikitudeSDK captureDeviceManager] focusDistance];
}

- (AVCaptureTorchMode)currentlyActiveTorchModeForCameraControlsModel:(WTCameraControlsModel *)cameraControlsModel
{
    return [[self.wikitudeSDK captureDeviceManager] torchMode];
}

@end
