//
//  WTExtendedTrackingViewController.m
//  Native Examples
//
//  Created by Andreas Schacherbauer on 28/07/15.
//  Copyright (c) 2015 Wikitude. All rights reserved.
//

#import "WTExtendedTrackingViewController.h"

#import <WikitudeNativeSDK/WikitudeNativeSDK.h>

#import "WikitudeLicense.h"
#import "StrokedRectangle.h"
#import "ExternalRenderer.h"
#import "WTTargetImageView.h"


@interface WTExtendedTrackingViewController () <WTWikitudeNativeSDKDelegate, WTImageTrackerDelegate>

@property (nonatomic, strong) WTWikitudeNativeSDK                           *wikitudeSDK;
@property (nonatomic, strong) WTImageTracker                                *imageTracker;
@property (nonatomic, strong) WTTargetCollectionResource                    *targetCollectionResource;

@property (nonatomic, strong) EAGLContext                                   *sharedWikitudeEAGLCameraContext;

@property (nonatomic, copy) WTWikitudeUpdateHandler                         wikitudeUpdateHandler;
@property (nonatomic, copy) WTWikitudeDrawHandler                           wikitudeDrawHandler;

@property (nonatomic, assign) BOOL                                          isTracking;

@property (nonatomic, strong) ExternalRenderer                              *renderer;
@property (nonatomic, strong) StrokedRectangle                              *renderableRectangle;
@property (nonatomic, strong) StrokedRectangle                              *extendedRenderableRectangle;

@property (nonatomic, strong) WTTargetImageView                             *targetImageView;

@end


@implementation WTExtendedTrackingViewController

#pragma mark UIViewController

- (void)viewDidLoad
{
    [super viewDidLoad];

    self.renderer = [[ExternalRenderer alloc] init];
    self.renderableRectangle = [[StrokedRectangle alloc] init];
    self.renderableRectangle.scale = 1.0f;
    self.extendedRenderableRectangle = [[StrokedRectangle alloc] init];
    self.extendedRenderableRectangle.scale = 2.5f;
    [self.extendedRenderableRectangle setColor:[UIColor colorWithRed:0.202 green:0.552 blue:1 alpha:1]];


    self.wikitudeSDK = [[WTWikitudeNativeSDK alloc] initWithRenderingMode:WTRenderingMode_External delegate:self];
    [self.wikitudeSDK setLicenseKey:kWTLicenseKey];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didReceiveApplicationWillResignActiveNotification:) name:UIApplicationWillResignActiveNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didReceiveApplicationDidBecomeActiveNotification:) name:UIApplicationDidBecomeActiveNotification object:nil];

    [self updateExtendedTrackingVisibility:NO];

    self.targetImageView = [[WTTargetImageView alloc] init];
    [self.view addSubview:self.targetImageView];

    NSArray<NSString *> *imageNames = @[@"target_iot"];
    [self.targetImageView setupViewWithImages:imageNames andTopLayoutGuide:self.topLayoutGuide];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];

    [self.renderer setupRenderingWithLayer:[self.eaglView eaglLayer] OpenGLESRenderingAPIs:[NSOrderedSet orderedSetWithObjects:RenderingAPI_OpenGL_ES_2, nil]];
    [self.renderer startRenderLoopWithRenderBlock:[self renderBlock]];

    [self.wikitudeSDK start:nil completion:^(BOOL isRunning, NSError * __nonnull error) {
        if ( !isRunning ) {
            NSLog(@"Wikitude SDK is not running. Reason: %@", [error localizedDescription]);
        }
        else
        {
            __weak typeof(self) weakSelf = self;
            
            NSURL *imageTrackerResourceURL = [[NSBundle mainBundle] URLForResource:@"iot_tracker" withExtension:@"wtc" subdirectory:@"Assets"];
            self.targetCollectionResource = [self.wikitudeSDK.trackerManager createTargetCollectionResourceFromURL:imageTrackerResourceURL completion:^(BOOL success, NSError * error) {
                if ( !success )
                {
                    NSLog(@"Failed to load URL resource. Reason: %@", [error localizedDescription]);
                }
                else
                {
                    weakSelf.imageTracker = [weakSelf.wikitudeSDK.trackerManager createImageTrackerFromTargetCollectionResource:weakSelf.targetCollectionResource delegate:weakSelf configuration:^(WTImageTrackerConfiguration *imageTrackerConfiguration) {
                        imageTrackerConfiguration.extendedTargets = @[@"*"];
                    }];
                }
            }];
        }
    }];

    [self.targetImageView show:YES];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];

    [self dismissNavigationBarPrompt];
    
    [self.wikitudeSDK stop];
    [self.renderableRectangle releaseProgram];
    [self.renderer stopRenderLoop];
    [self.renderer teardownRendering];
    
    _sharedWikitudeEAGLCameraContext = nil;
    
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)viewWillTransitionToSize:(CGSize)size withTransitionCoordinator:(id<UIViewControllerTransitionCoordinator>)coordinator
{
    [super viewWillTransitionToSize:size withTransitionCoordinator:coordinator];

    [self.wikitudeSDK shouldTransitionToSize:size withTransitionCoordinator:coordinator];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Public Methods
- (IBAction)toggleExtendedTracking:(UIButton *)sender
{
    [self.imageTracker stopExtendedTracking];
    [self updateExtendedTrackingVisibility:NO];
}

#pragma mark - Private Methods
- (void)dismissNavigationBarPrompt
{
    if ( nil != self.navigationItem.prompt )
    {
        [self.navigationController.navigationBar setBarTintColor:[UIColor whiteColor]];
        [self.navigationItem setPrompt:nil];
    }
}

- (void)updateExtendedTrackingVisibility:(BOOL)visible
{
    if ( visible )
    {
        if ( _stopExtendedTrackingButton.hidden ) {
            _stopExtendedTrackingButton.hidden = NO;
        }
    }
    else
    {
        if ( !_stopExtendedTrackingButton.hidden ) {
            _stopExtendedTrackingButton.hidden = YES;
        }
    }
}

#pragma mark - ExternalRenderer render loop
- (ExternalRenderBlock)renderBlock
{
    return ^ (CADisplayLink *displayLink) {
        if ( self.wikitudeUpdateHandler
            &&
            self.wikitudeDrawHandler )
        {
            self.wikitudeUpdateHandler();
            self.wikitudeDrawHandler();
        }

        [self.renderer bindBuffer];

        if ( _isTracking )
        {
            [self.renderableRectangle drawInContext:[self.renderer internalContext]];
            [self.extendedRenderableRectangle drawInContext:[self.renderer internalContext]];
        }
    };
}

#pragma mark - Notifications
- (void)didReceiveApplicationWillResignActiveNotification:(NSNotification *)notification
{
    [self.wikitudeSDK stop];

    [self.renderer stopRenderLoop];
}

- (void)didReceiveApplicationDidBecomeActiveNotification:(NSNotification *)notification
{
    [self.renderer startRenderLoopWithRenderBlock:[self renderBlock]];

    [self.wikitudeSDK start:nil completion:^(BOOL isRunning, NSError * __nonnull error) {
        if ( !isRunning ) {
            NSLog(@"Wikitude SDK is not running. Reason: %@", [error localizedDescription]);
        }
    }];
}

#pragma mark - Delegation
#pragma mark WTWikitudeNativeSDKDelegte
- (void)wikitudeNativeSDK:(WTWikitudeNativeSDK * __nonnull)wikitudeNativeSDK didCreatedExternalUpdateHandler:(WTWikitudeUpdateHandler __nonnull)updateHandler
{
    self.wikitudeUpdateHandler = updateHandler;
}

- (void)wikitudeNativeSDK:(WTWikitudeNativeSDK * __nonnull)wikitudeNativeSDK didCreatedExternalDrawHandler:(WTWikitudeDrawHandler __nonnull)drawHandler
{
    self.wikitudeDrawHandler = drawHandler;
}

- (EAGLContext *)eaglContextForVideoCameraInWikitudeNativeSDK:(WTWikitudeNativeSDK * __nonnull)wikitudeNativeSDK
{
    if (!_sharedWikitudeEAGLCameraContext )
    {
        EAGLContext *rendererContext = [self.renderer internalContext];
        self.sharedWikitudeEAGLCameraContext = [[EAGLContext alloc] initWithAPI:[rendererContext API] sharegroup:[rendererContext sharegroup]];
    }
    return self.sharedWikitudeEAGLCameraContext;
}

- (CGRect)eaglViewSizeForExternalRenderingInWikitudeNativeSDK:(WTWikitudeNativeSDK * __nonnull)wikitudeNativeSDK
{
    return self.eaglView.bounds;
}

- (void)wikitudeNativeSDK:(WTWikitudeNativeSDK * __nonnull)wikitudeNativeSDK didEncounterInternalError:(NSError * __nonnull)error
{
    NSLog(@"Internal Wikitude SDK error encounterd. %@", [error localizedDescription]);
}

#pragma mark WTClientTrackerDelegate

- (void)imageTracker:(nonnull WTImageTracker *)imageTracker didRecognizeImage:(nonnull WTImageTarget *)recognizedTarget
{
    NSLog(@"recognized target '%@'", [recognizedTarget name]);
    _isTracking = YES;
    [self.targetImageView hide:YES];

    [self updateExtendedTrackingVisibility:YES];
}

- (void)imageTracker:(nonnull WTImageTracker *)imageTracker didTrackImage:(nonnull WTImageTarget *)trackedTarget
{
    [self.renderableRectangle setProjectionMatrix:trackedTarget.projection];
    [self.renderableRectangle setModelViewMatrix:trackedTarget.modelView];
    
    [self.extendedRenderableRectangle setProjectionMatrix:trackedTarget.projection];
    [self.extendedRenderableRectangle setModelViewMatrix:trackedTarget.modelView];
}

- (void)imageTracker:(nonnull WTImageTracker *)imageTracker didUpdateExtendedTrackingQualityForTarget:(nonnull NSString *)extendedTarget fromQuality:(WTExtendedTrackingQuality)oldTrackingQuality toQuality:(WTExtendedTrackingQuality)newTrackingQuality
{
    UIColor *extendedTrackingQualityBarTintColor = nil;
    NSString *extendedTrackingQualityText = nil;

    if ( WTExtendedTrackingQuality_Bad == newTrackingQuality )
    {
        extendedTrackingQualityBarTintColor = [UIColor colorWithRed:1 green:0.204 blue:0.125 alpha:1];
        extendedTrackingQualityText = @"Bad";
    }
    else if ( WTExtendedTrackingQuality_Average == newTrackingQuality )
    {
        extendedTrackingQualityBarTintColor = [UIColor colorWithRed:1 green:0.851 blue:0 alpha:1];
        extendedTrackingQualityText = @"Average";
    }
    else
    {
        extendedTrackingQualityBarTintColor = [UIColor colorWithRed:0.420 green:1 blue:0 alpha:1];
        extendedTrackingQualityText = @"Good";
    }

    if ( extendedTrackingQualityBarTintColor
        &&
         extendedTrackingQualityText)
    {
        [self.navigationController.navigationBar setBarTintColor:extendedTrackingQualityBarTintColor];
        [self.navigationItem setPrompt:[NSString stringWithFormat:@"Extended tracking quality: %@", extendedTrackingQualityText]];
    }
}

- (void)imageTracker:(nonnull WTImageTracker *)imageTracker didLoseImage:(nonnull WTImageTarget *)lostTarget
{
    NSLog(@"lost target '%@'", [lostTarget name]);
    _isTracking = NO;

    [self dismissNavigationBarPrompt];
}


- (void)imageTrackerDidLoadTargets:(WTImageTracker *)imageTracker
{
    NSLog(@"Image tracker loaded");
}

- (void)imageTracker:(nonnull WTImageTracker *)imageTracker didFailToLoadTargets:(nonnull NSError *)error
{
    NSLog(@"Unable to load image tracker. Reason: %@", [error localizedDescription]);
}

@end
