//
//  WTBarcodePluginViewController.m
//  Native Examples
//
//  Created by Andreas Schacherbauer on 29/07/15.
//  Copyright (c) 2015 Wikitude. All rights reserved.
//

#import "WTBarcodePluginViewController.h"

#import <AVFoundation/AVCaptureSession.h>
#import <WikitudeNativeSDK/WikitudeNativeSDK.h>

#import "WikitudeLicense.h"
#import "BarcodePlugin.h"
#import "StrokedRectangle.h"
#import "ExternalRenderer.h"
#import "WTTargetImageView.h"


@interface WTBarcodePluginViewController () <WTWikitudeNativeSDKDelegate, WTImageTrackerDelegate>

@property (nonatomic, strong) WTWikitudeNativeSDK                           *wikitudeSDK;
@property (nonatomic, strong) WTTargetCollectionResource                    *targetCollectionResource;
@property (nonatomic, strong) WTImageTracker                                *imageTracker;

@property (nonatomic, strong) EAGLContext                                   *sharedWikitudeEAGLCameraContext;

@property (nonatomic, copy) WTWikitudeUpdateHandler                         wikitudeUpdateHandler;
@property (nonatomic, copy) WTWikitudeDrawHandler                           wikitudeDrawHandler;

@property (nonatomic, assign) BOOL                                          isTracking;

@property (nonatomic, strong) ExternalRenderer                              *renderer;
@property (nonatomic, strong) StrokedRectangle                              *renderableRectangle;

@property (nonatomic, assign) std::shared_ptr<BarcodePlugin>                barcodePlugin;

@property (nonatomic, strong) WTTargetImageView                             *targetImageView;

@end

@implementation WTBarcodePluginViewController

#pragma mark UIViewController

- (void)viewDidLoad
{
    [super viewDidLoad];

    self.renderer = [[ExternalRenderer alloc] init];
    self.renderableRectangle = [[StrokedRectangle alloc] init];    
    
    self.wikitudeSDK = [[WTWikitudeNativeSDK alloc] initWithRenderingMode:WTRenderingMode_External delegate:self];
    [self.wikitudeSDK setLicenseKey:kWTLicenseKey];

    _barcodePlugin = std::make_shared<BarcodePlugin>(640, 480, self);
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didReceiveApplicationWillResignActiveNotification:) name:UIApplicationWillResignActiveNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didReceiveApplicationDidBecomeActiveNotification:) name:UIApplicationDidBecomeActiveNotification object:nil];

    self.targetImageView = [[WTTargetImageView alloc] init];
    [self.view addSubview:self.targetImageView];

    NSArray<NSString *> *imageNames = @[@"surfer", @"bike"];
    [self.targetImageView setupViewWithImages:imageNames andTopLayoutGuide:self.topLayoutGuide];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];

    [self.renderer setupRenderingWithLayer:[self.eaglView eaglLayer] OpenGLESRenderingAPIs:[NSOrderedSet orderedSetWithObjects:RenderingAPI_OpenGL_ES_2, nil]];
    [self.renderer startRenderLoopWithRenderBlock:[self renderBlock]];

    [self.wikitudeSDK start:nil completion:^(BOOL isRunning, NSError * __nonnull error) {
        if ( !isRunning ) {
            NSLog(@"Wikitude SDK is not running. Reason: %@", [error localizedDescription]);
        }
        else
        {
            __weak typeof(self) weakSelf = self;
            
            NSURL *imageTrackerResourceURL = [[NSBundle mainBundle] URLForResource:@"magazine" withExtension:@"wtc" subdirectory:@"Assets"];
            self.targetCollectionResource = [self.wikitudeSDK.trackerManager createTargetCollectionResourceFromURL:imageTrackerResourceURL completion:^(BOOL success, NSError * _Nullable error) {
                if ( !success )
                {
                    NSLog(@"Failed to load URL resource. Reason: %@", [error localizedDescription]);
                }
                else
                {
                    weakSelf.imageTracker = [weakSelf.wikitudeSDK.trackerManager createImageTrackerFromTargetCollectionResource:weakSelf.targetCollectionResource delegate:weakSelf configuration:nil];
                }
            }];

            NSError *error = nil;
            BOOL pluginRegistered = [self.wikitudeSDK registerPlugin:_barcodePlugin error:&error];
            if ( !pluginRegistered )
            {
                NSLog(@"Unable to register plugin '%@'. Error: %@", [NSString stringWithUTF8String:_barcodePlugin->getIdentifier().c_str()], [error localizedDescription]);
            }
        }
    }];

    [self.navigationItem setPrompt:@"Scanning for barcodes..."];
    [self.targetImageView show:YES];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];

    [self.navigationItem setPrompt:nil];

    [self.wikitudeSDK removePlugin:_barcodePlugin];
    [self.wikitudeSDK stop];
    [self.renderableRectangle releaseProgram];
    [self.renderer stopRenderLoop];
    [self.renderer teardownRendering];
    
    _sharedWikitudeEAGLCameraContext = nil;
    
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)viewWillTransitionToSize:(CGSize)size withTransitionCoordinator:(id<UIViewControllerTransitionCoordinator>)coordinator
{
    [super viewWillTransitionToSize:size withTransitionCoordinator:coordinator];

    [self.wikitudeSDK shouldTransitionToSize:size withTransitionCoordinator:coordinator];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Public Methods
- (void)presentBarCodeResult:(NSString *)barcodeScanResult
{
    NSString *uiBarcodePrompt = nil;
    if ( barcodeScanResult )
    {
        uiBarcodePrompt = [NSString stringWithFormat:@"Found '%@'", barcodeScanResult];
    }
    else
    {
        uiBarcodePrompt = @"Scanning for barcodes...";
    }

    dispatch_async(dispatch_get_main_queue(), ^{
        [self.navigationItem setPrompt:uiBarcodePrompt];
    });
}

#pragma mark - ExternalRenderer render loop
- (ExternalRenderBlock)renderBlock
{
    return ^ (CADisplayLink *displayLink) {
        if ( self.wikitudeUpdateHandler
            &&
            self.wikitudeDrawHandler )
        {
            self.wikitudeUpdateHandler();
            self.wikitudeDrawHandler();
        }

        [self.renderer bindBuffer];

        if ( _isTracking )
        {
            [self.renderableRectangle drawInContext:[self.renderer internalContext]];
        }
    };
}

#pragma mark - Notifications
- (void)didReceiveApplicationWillResignActiveNotification:(NSNotification *)notification
{
    [self.wikitudeSDK removePlugin:_barcodePlugin];
    [self.wikitudeSDK stop];

    [self.renderer stopRenderLoop];
}

- (void)didReceiveApplicationDidBecomeActiveNotification:(NSNotification *)notification
{
    [self.renderer startRenderLoopWithRenderBlock:[self renderBlock]];

    [self.wikitudeSDK start:nil completion:^(BOOL isRunning, NSError * __nonnull error) {
        if ( !isRunning ) {
            NSLog(@"Wikitude SDK is not running. Reason: %@", [error localizedDescription]);
        }
        else
        {
            NSError *error = nil;
            BOOL pluginRegistered = [self.wikitudeSDK registerPlugin:_barcodePlugin error:&error];
            if ( !pluginRegistered )
            {
                NSLog(@"Unable to register plugin '%@'. Error: %@", [NSString stringWithUTF8String:_barcodePlugin->getIdentifier().c_str()], [error localizedDescription]);
            }
        }
    }];
}

#pragma mark - Delegation
#pragma mark WTWikitudeNativeSDKDelegte
- (void)wikitudeNativeSDK:(WTWikitudeNativeSDK * __nonnull)wikitudeNativeSDK didCreatedExternalUpdateHandler:(WTWikitudeUpdateHandler __nonnull)updateHandler
{
    self.wikitudeUpdateHandler = updateHandler;
}

- (void)wikitudeNativeSDK:(WTWikitudeNativeSDK * __nonnull)wikitudeNativeSDK didCreatedExternalDrawHandler:(WTWikitudeDrawHandler __nonnull)drawHandler
{
    self.wikitudeDrawHandler = drawHandler;
}

- (EAGLContext *)eaglContextForVideoCameraInWikitudeNativeSDK:(WTWikitudeNativeSDK * __nonnull)wikitudeNativeSDK
{
    if (!_sharedWikitudeEAGLCameraContext )
    {
        EAGLContext *rendererContext = [self.renderer internalContext];
        self.sharedWikitudeEAGLCameraContext = [[EAGLContext alloc] initWithAPI:[rendererContext API] sharegroup:[rendererContext sharegroup]];
    }
    return self.sharedWikitudeEAGLCameraContext;
}

- (CGRect)eaglViewSizeForExternalRenderingInWikitudeNativeSDK:(WTWikitudeNativeSDK * __nonnull)wikitudeNativeSDK
{
    return self.eaglView.bounds;
}

- (void)wikitudeNativeSDK:(WTWikitudeNativeSDK * __nonnull)wikitudeNativeSDK didEncounterInternalError:(NSError * __nonnull)error
{
    NSLog(@"Internal Wikitude SDK error encounterd. %@", [error localizedDescription]);
}

#pragma mark WTImageTrackerDelegate

- (void)imageTracker:(nonnull WTImageTracker *)imageTracker didRecognizeImage:(nonnull WTImageTarget *)recognizedTarget
{
    NSLog(@"recognized target '%@'", [recognizedTarget name]);
    _isTracking = YES;
    [self.targetImageView hide:YES];
}

- (void)imageTracker:(nonnull WTImageTracker *)imageTracker didTrackImage:(nonnull WTImageTarget *)trackedTarget
{
    [self.renderableRectangle setProjectionMatrix:trackedTarget.projection];
    [self.renderableRectangle setModelViewMatrix:trackedTarget.modelView];
}

- (void)imageTracker:(nonnull WTImageTracker *)imageTracker didLoseImage:(nonnull WTImageTarget *)lostTarget
{
    NSLog(@"lost target '%@'", [lostTarget name]);
    _isTracking = NO;
}

- (void)imageTrackerDidLoadTargets:(nonnull WTImageTracker *)imageTracker
{
    NSLog(@"Image tracker loaded");
}

- (void)imageTracker:(nonnull WTImageTracker *)imageTracker didFailToLoadTargets:(nonnull NSError *)error
{
    NSLog(@"Unable to load image tracker. Reason: %@", [error localizedDescription]);
}

@end
