//
//  WTTargetImageView.m
//  Native Examples
//
//  Created by Max Meraner on 01/02/2017.
//  Copyright © 2017 Wikitude. All rights reserved.
//

#import "WTTargetImageView.h"


@interface WTTargetImageView ()

@property (nonatomic, strong) NSLayoutConstraint                            *targetImageViewTopConstraint;

@property (nonatomic, assign) CGFloat                                       viewHeight;

@end

@implementation WTTargetImageView

- (void)setupViewWithImages:(NSArray<NSString *> *)imageNames andTopLayoutGuide:(id<UILayoutSupport>)topLayoutGuide
{
    if (self.superview != nil)
    {
        self.viewHeight = 100.f;
        
        UIColor *lightGrayColorWithAlpha = [[UIColor lightGrayColor] colorWithAlphaComponent:0.5f];
        self.backgroundColor = lightGrayColorWithAlpha;
        self.translatesAutoresizingMaskIntoConstraints = NO;
        
        NSDictionary *views = NSDictionaryOfVariableBindings(self);
        [self.superview addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"|[self]|" options:0 metrics:nil views:views]];
        
        self.targetImageViewTopConstraint = [NSLayoutConstraint
                                             constraintWithItem:self
                                             attribute:NSLayoutAttributeTop
                                             relatedBy:NSLayoutRelationEqual
                                             toItem:topLayoutGuide
                                             attribute:NSLayoutAttributeBottom
                                             multiplier:1.f
                                             constant:-self.viewHeight];
        [self.superview addConstraint:self.targetImageViewTopConstraint];
        
        NSLayoutConstraint *targetImageViewHeightConstraint = [NSLayoutConstraint
                                                               constraintWithItem:self
                                                               attribute:NSLayoutAttributeHeight
                                                               relatedBy:NSLayoutRelationEqual
                                                               toItem:nil
                                                               attribute:NSLayoutAttributeNotAnAttribute
                                                               multiplier:1.f
                                                               constant:self.viewHeight];
        [self addConstraint:targetImageViewHeightConstraint];
        
        [self setupContainerViewWithImages:imageNames];
    }
    else
    {
        NSLog(@"WTTargetImageView is not connected to a superview. Please call setupViewWithImages:andTopLayoutGuide: after adding the WTTargetImageView to a superview.");
    }
}

/* 
    This function creates a container UIView and adds a UIImageView for each image in the received images array.
    It further sets up the autoLayout behavior of every element in the container UIView and also adds a label if it fits the view.
    The images are added from left to right.
*/
- (void)setupContainerViewWithImages:(NSArray<NSString *> *)imageNames
{
    float imageWidth = 60.f;
    float xOffset = 10.f;
    
    UIView *containerView = [[UIView alloc] init];
    containerView.translatesAutoresizingMaskIntoConstraints = NO;
    [self addSubview:containerView];
    
    UILabel *label = [[UILabel alloc] init];
    label.translatesAutoresizingMaskIntoConstraints = NO;
    label.text = (imageNames.count == 1) ? @"Target image:" : @"Target images:";
    [label sizeToFit];
    
    float containerWidth = label.intrinsicContentSize.width + (imageWidth + xOffset) * imageNames.count;
    UIView *lastAddedView = [[UIView alloc] init];
    
    BOOL omitLabel = self.superview.bounds.size.width < containerWidth;
    if (omitLabel)
    {
        containerWidth -= label.intrinsicContentSize.width + xOffset;
    }
    else
    {
        [containerView addSubview:label];
        
        UIView *superview = label.superview;
        
        NSDictionary *views = NSDictionaryOfVariableBindings(label, superview);
        [label.superview addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"|[label]" options:0 metrics:nil views:views]];
        [label.superview addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"H:[superview]-(<=1)-[label]" options:NSLayoutFormatAlignAllCenterY metrics:nil views:views]];
        
        lastAddedView = label;
    }
    
    for (NSUInteger i = 0; i < imageNames.count; i++)
    {
        UIImageView *imageView = [[UIImageView alloc] initWithImage:[UIImage imageNamed:[imageNames objectAtIndex:i]]];
        imageView.translatesAutoresizingMaskIntoConstraints = NO;
        imageView.contentMode = UIViewContentModeScaleAspectFit;
        [containerView addSubview:imageView];
        
        if (i == 0 && omitLabel)
        {
            NSLayoutConstraint *imageViewLeftConstraint = [NSLayoutConstraint
                                                           constraintWithItem:imageView
                                                           attribute:NSLayoutAttributeLeft
                                                           relatedBy:NSLayoutRelationEqual
                                                           toItem:imageView.superview
                                                           attribute:NSLayoutAttributeLeft
                                                           multiplier:1.f
                                                           constant:0.f];
            [imageView.superview addConstraint:imageViewLeftConstraint];
        }
        else
        {
            NSLayoutConstraint *imageViewTrailingConstraint = [NSLayoutConstraint
                                                               constraintWithItem:imageView
                                                               attribute:NSLayoutAttributeLeading
                                                               relatedBy:NSLayoutRelationEqual
                                                               toItem:lastAddedView
                                                               attribute:NSLayoutAttributeTrailing
                                                               multiplier:1.f
                                                               constant:xOffset];
            [imageView.superview addConstraint:imageViewTrailingConstraint];
        }
        
        NSLayoutConstraint *imageViewTopConstraint = [NSLayoutConstraint
                                                      constraintWithItem:imageView.superview
                                                      attribute:NSLayoutAttributeTop
                                                      relatedBy:NSLayoutRelationEqual
                                                      toItem:imageView
                                                      attribute:NSLayoutAttributeTop
                                                      multiplier:1.f
                                                      constant:-5.f];
        
        NSLayoutConstraint *imageViewBottomConstraint = [NSLayoutConstraint
                                                         constraintWithItem:imageView.superview
                                                         attribute:NSLayoutAttributeBottom
                                                         relatedBy:NSLayoutRelationEqual
                                                         toItem:imageView
                                                         attribute:NSLayoutAttributeBottom
                                                         multiplier:1.f
                                                         constant:5.f];
        
        NSLayoutConstraint *imageViewWidthConstraint = [NSLayoutConstraint
                                                        constraintWithItem:imageView
                                                        attribute:NSLayoutAttributeWidth
                                                        relatedBy:NSLayoutRelationGreaterThanOrEqual
                                                        toItem:nil
                                                        attribute:NSLayoutAttributeNotAnAttribute
                                                        multiplier:1.f
                                                        constant:imageWidth];
        
        [imageView.superview addConstraint:imageViewTopConstraint];
        [imageView.superview addConstraint:imageViewBottomConstraint];
        [imageView addConstraint:imageViewWidthConstraint];
        
        lastAddedView = imageView;
    }
    
    UIView *superview = containerView.superview;
    NSDictionary *views = NSDictionaryOfVariableBindings(containerView, superview);
    
    [containerView.superview addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"V:|[containerView(==superview)]|" options:0 metrics:nil views:views]];
    
    NSLayoutConstraint *containerViewCenterXConstraint = [NSLayoutConstraint
                                                          constraintWithItem:containerView
                                                          attribute:NSLayoutAttributeCenterX
                                                          relatedBy:NSLayoutRelationEqual
                                                          toItem:containerView.superview
                                                          attribute:NSLayoutAttributeCenterX
                                                          multiplier:1.f
                                                          constant:0.f];
    [containerView.superview addConstraint:containerViewCenterXConstraint];
    
    NSLayoutConstraint *containerViewWidthConstraint = [NSLayoutConstraint
                                                        constraintWithItem:containerView
                                                        attribute:NSLayoutAttributeWidth
                                                        relatedBy:NSLayoutRelationEqual
                                                        toItem:nil
                                                        attribute:NSLayoutAttributeNotAnAttribute
                                                        multiplier:1.f
                                                        constant:containerWidth];
    [containerView addConstraint:containerViewWidthConstraint];
}

- (void)show:(BOOL)animated
{
    if (animated)
    {
        if (self.targetImageViewTopConstraint.constant == -self.viewHeight)
        {
            self.alpha = 1.f;
            [self.superview layoutIfNeeded];
            self.targetImageViewTopConstraint.constant = 0;
            [UIView animateWithDuration:0.5
                             animations:^{
                                 [self.superview layoutIfNeeded];
                             }];
        }
    }
    else
    {
        if (self.alpha == 0.f)
        {
            self.targetImageViewTopConstraint.constant = 0;
            self.alpha = 1.f;
        }
    }
}

- (void)hide:(BOOL)animated
{
    if (animated)
    {
        if (self.targetImageViewTopConstraint.constant == 0)
        {
            [self.superview layoutIfNeeded];
            self.targetImageViewTopConstraint.constant = -self.viewHeight;
            [UIView animateWithDuration:0.5
                             animations:^{
                                 [self.superview layoutIfNeeded];
                             } completion:^(BOOL finished) {
                                 self.alpha = 0.f;
                             }];
        }
    }
    else
    {
        if (self.alpha == 1.f)
        {
            self.targetImageViewTopConstraint.constant = -self.viewHeight;
            self.alpha = 0.f;
        }
    }
}

@end
